import requests
from packaging import version
from tnscm._version import __version__ as current_version
from tnscm import __about__


def check_for_update():

    PACKAGE_NAME = __about__.__package_name__

    try:
        response = requests.get(
            f"https://pypi.org/pypi/{PACKAGE_NAME}/json", timeout=1.5
        )
        response.raise_for_status()
        latest = response.json()["info"]["version"]
        read_more = (
            f"> Read more:\n"
            f"> https://limberduck.org/en/latest/tools/{PACKAGE_NAME}\n"
            f"> https://github.com/LimberDuck/{PACKAGE_NAME}\n"
            f"> https://github.com/LimberDuck/{PACKAGE_NAME}/releases"
        )
        if version.parse(latest) > version.parse(current_version):
            print(
                f"\n> A new version of {PACKAGE_NAME} is available: {latest} (you have {current_version})"
            )
            print(f"> Update with: pip install -U {PACKAGE_NAME}\n")
            print(read_more)
        elif version.parse(latest) == version.parse(current_version):
            print(
                f"\n> You are using the latest version of {PACKAGE_NAME}: {current_version}\n"
            )
            print(read_more)
        else:
            print(
                f"\n> You are using a pre-release version of {PACKAGE_NAME}: {current_version}"
            )
            print(f"> Latest released version of {PACKAGE_NAME}: {latest}\n")
            print(read_more)
    except requests.exceptions.ConnectionError as e:
        print("> Could not check for updates: Connection error.\n")
        print(e)
    except Exception as e:
        print("> Could not check for updates:\n")
        print(e)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt") as f:
    required = f.read().splitlines()

about = {}
with open("tnscm/_version.py") as f:
    exec(f.read(), about)

setuptools.setup(
    name="tnscm",
    version=about["__version__"],
    license="MIT",
    author="Damian Krawczyk",
    author_email="damian.krawczyk@limberduck.org",
    description="TNSCM (Tenable Nessus CLI Manager) by LimberDuck",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/LimberDuck/tnscm",
    packages=setuptools.find_packages(),
    install_requires=required,
    entry_points={"console_scripts": ["tnscm = tnscm.__main__:main"]},
    classifiers=[
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Environment :: Console",
    ],
)



# -*- coding: utf-8 -*-
"""
TNSCM by LimberDuck (pronounced *ˈlɪm.bɚ dʌk*) is a CLI tool which
enables you to perform certain actions on Tenable Nessus by (C) Tenable,
Inc. via Nessus API.
Copyright (C) 2025 Damian Krawczyk

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

__all__ = [
    "__title__",
    "__icon__",
    "__summary__",
    "__uri__",
    "__version__",
    "__release_date__",
    "__author__",
    "__email__",
    "__license_name__",
    "__license_link__",
    "__copyright__",
]

__title__ = "TNSCM by LimberDuck"
__package_name__ = "tnscm"
__icon__ = "LimberDuck-tnscm.ico"
__summary__ = (
    "TNSCM by LimberDuck (pronounced *ˈlɪm.bɚ dʌk*) is a CLI tool which"
    "enables you to perform certain actions on Tenable Nessus by (C) Tenable,"
    "Inc. via Nessus API."
)
__uri__ = "https://github.com/LimberDuck"
__version__ = "0.0.6"
__release_date__ = "2025.09.01"
__author__ = "Damian Krawczyk"
__email__ = "damian.krawczyk@limberduck.org"
__license_name__ = "GNU GPLv3"
__license_link__ = "https://www.gnu.org/licenses/gpl-3.0.en.html"
__copyright__ = "\N{COPYRIGHT SIGN} 2021-2025 by %s" % __author__



import requests
import json
import certstore
import urllib3
import sys
import datetime


class TnsApi:

    def __init__(self, host="127.0.0.1", port=443, insecure=None):
        """
        :param host: address to Nessus API `127.0.0.1`
        :param port: port to Nessus API `443`
        :param insecure: if True perform insecure SSL connections and transfers
        """
        self.host = host
        self.port = port
        self.verify = certstore.ca_bundle

        if not insecure:
            self.verify = certstore.ca_bundle
        else:
            self.verify = False
            urllib3.disable_warnings()

        self._token = ""

    def build_url(self, resource):
        url = "{}://{}:{}".format("https", self.host, self.port)
        return "{}{}".format(url, resource)

    def connect(self, method, resource, data=None):

        headers = {
            "X-Cookie": "token={0}".format(self._token),
            "content-type": "application/json",
        }

        data = json.dumps(data)

        if method == "POST":
            r = requests.post(
                self.build_url(resource), data=data, headers=headers, verify=self.verify
            )
        elif method == "PUT":
            r = requests.put(
                self.build_url(resource), data=data, headers=headers, verify=self.verify
            )
        elif method == "DELETE":
            r = requests.delete(
                self.build_url(resource), data=data, headers=headers, verify=self.verify
            )
        else:
            r = requests.get(
                self.build_url(resource), data=data, headers=headers, verify=self.verify
            )

        if r.status_code == 401:
            print("Response code: {}".format(r.status_code))
            print("Unauthorized.")
            sys.exit()
        if r.status_code == 500:
            print("Response code: {}".format(r.status_code))
            print("Internal Server Error.")
            sys.exit()
        if r.status_code == 503:
            print("Response code: {}".format(r.status_code))
            print("Service Unavailable.")
            sys.exit()

        if method == "POST":
            return r.json()
        elif method == "PUT":
            if r is None:
                return None
        elif method == "DELETE":
            return r
        else:
            if "download" in resource:
                return r.content
            else:
                return r.json()

    def login(self, usr, pwd):
        """
        Login to Nessus.
        """

        login = {"username": usr, "password": pwd}
        data = self.connect("POST", "/session", data=login)
        self._token = data["token"]
        return self._token

    def logout(self):
        """
        Logout of Nessus.
        """
        self.connect("DELETE", "/session")

    def session_get(self):
        data = self.connect("GET", "/session")
        return data

    def server_status_get(self):
        data = self.connect("GET", "/server/status")["status"]
        return data

    def server_properties_get(self):
        data = self.connect("GET", "/server/properties")
        return data

    def policies_get(self):
        data = self.connect("GET", "/policies")["policies"]
        return data

    def policies_delete(self, id):
        data = self.connect("DELETE", "/policies/{0}".format(id))
        return data

    def users_get(self):
        data = self.connect("GET", "/users")["users"]
        return data

    def folders_get(self):
        data = self.connect("GET", "/folders")
        return data

    def scans_get(self):
        data = self.connect("GET", "/scans")["scans"]
        return data

    def scan_delete(self, id):
        data = self.connect("DELETE", "/scans/{0}".format(id))
        return data

    def plugins_families_get(self):
        data = self.connect("GET", "/plugins/families")["families"]
        return data

    def settings_advanced_get(self):
        data = self.connect("GET", "/settings/advanced")["preferences"]
        return data
